/* -*- Mode: IDL; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface nsIContentViewer;

[scriptable, uuid(ea2ce7a0-5c3d-11d4-90c2-0050041caf44)]
interface nsIContentViewerContainer : nsISupports {
	void embed(in nsIContentViewer aDocViewer, in string aCommand, in nsISupports aExtraInfo);

  /**
   * Allows the PrintEngine to make this call on 
   * an internal interface to the DocShell
   */
  void setIsPrinting(in boolean aIsPrinting);
};
